@@ -1,21 +1,35 @@
+/* eslint-disable import/no-unresolved */
 /* global gc */
 
-const { readFileSync, writeFileSync } = require('fs');
-const path = require('path');
-const colorette = require('colorette');
-const prettyBytes = require('pretty-bytes');
-const rollup = require('../dist/rollup.js');
-const { loadPerfConfig, targetDir } = require('./load-perf-config');
+import { readFileSync, writeFileSync } from 'node:fs';
+import { argv, chdir, cwd, exit } from 'node:process';
+import { fileURLToPath } from 'node:url';
+import { createColors } from 'colorette';
+import prettyBytes from 'pretty-bytes';
+import { loadConfigFile } from '../dist/loadConfigFile.js';
+import { rollup } from '../dist/rollup.js';
+import { findConfigFileName } from './find-config.js';
 
-const initialDir = process.cwd();
-const perfFile = path.resolve(targetDir, 'rollup.perf.json');
+const initialDirectory = cwd();
+const targetDirectory = fileURLToPath(new URL('../perf', import.meta.url).href);
+const perfFile = fileURLToPath(new URL('../perf/rollup.perf.json', import.meta.url).href);
+const { bold, underline, cyan, red, green } = createColors();
+const MIN_ABSOLUTE_TIME_DEVIATION = 10;
+const RELATIVE_DEVIATION_FOR_COLORING = 5;
+
+chdir(targetDirectory);
+const configFile = await findConfigFileName(targetDirectory);
+const configs = await loadConfigFile(
+	configFile,
+	configFile.endsWith('.ts') ? { configPlugin: 'typescript' } : {}
+);
 
 let numberOfRunsToAverage = 6;
 let numberOfDiscardedResults = 3;
-if (process.argv.length >= 3) {
-	numberOfRunsToAverage = Number.parseInt(process.argv[2]);
-	if (process.argv.length >= 4) {
-		numberOfDiscardedResults = Number.parseInt(process.argv[3]);
+if (argv.length >= 3) {
+	numberOfRunsToAverage = Number.parseInt(argv[2]);
+	if (argv.length >= 4) {
+		numberOfDiscardedResults = Number.parseInt(argv[3]);
 	}
 }
 if (!(numberOfDiscardedResults >= 0) || !(numberOfDiscardedResults < numberOfRunsToAverage)) {
@@ -24,16 +38,18 @@ if (!(numberOfDiscardedResults >= 0) || !(numberOfDiscardedResults < numberOfRun
 			'Usage: "npm run perf [<number of runs> [<number of discarded results>]]"\n' +
 			'where 0 <= <number of discarded results> < <number of runs>'
 	);
-	process.exit(1);
+	exit(1);
 }
 console.info(
-	colorette.bold(
-		`Calculating the average of ${colorette.cyan(
-			numberOfRunsToAverage
-		)} runs discarding the ${colorette.cyan(numberOfDiscardedResults)} largest results.\n`
+	bold(
+		`Calculating the average of ${cyan(numberOfRunsToAverage)} runs discarding the ${cyan(
+			numberOfDiscardedResults
+		)} largest results.\n`
 	) + 'Run "npm run perf <number of runs> <number of discarded results>" to change that.'
 );
 
+await calculatePrintAndPersistTimings(configs.options[0], await getExistingTimings());
+
 function getSingleAverage(times, runs, discarded) {
 	const actualDiscarded = Math.min(discarded, runs - 1);
 	return (
@@ -48,7 +64,7 @@ function getSingleAverage(times, runs, discarded) {
 
 function getAverage(accumulatedMeasurements, runs, discarded) {
 	const average = {};
-	Object.keys(accumulatedMeasurements).forEach(label => {
+	for (const label of Object.keys(accumulatedMeasurements)) {
 		average[label] = {
 			memory: getSingleAverage(
 				accumulatedMeasurements[label].map(timing => timing[2]),
@@ -61,15 +77,15 @@ function getAverage(accumulatedMeasurements, runs, discarded) {
 				discarded
 			)
 		};
-	});
+	}
 	return average;
 }
 
 async function calculatePrintAndPersistTimings(config, existingTimings) {
 	const timings = await buildAndGetTimings(config);
-	Object.keys(timings).forEach(label => {
+	for (const label of Object.keys(timings)) {
 		timings[label] = [timings[label]];
-	});
+	}
 	for (let currentRun = 1; currentRun < numberOfRunsToAverage; currentRun++) {
 		const numberOfLinesToClear = printMeasurements(
 			getAverage(timings, currentRun, numberOfDiscardedResults),
@@ -79,13 +95,13 @@ async function calculatePrintAndPersistTimings(config, existingTimings) {
 		console.info(`Completed run ${currentRun}.`);
 		const currentTimings = await buildAndGetTimings(config);
 		clearLines(numberOfLinesToClear);
-		Object.keys(timings).forEach(label => {
+		for (const label of Object.keys(timings)) {
 			if (!currentTimings.hasOwnProperty(label)) {
 				delete timings[label];
 			} else {
 				timings[label].push(currentTimings[label]);
 			}
-		});
+		}
 	}
 	const averageTimings = getAverage(timings, numberOfRunsToAverage, numberOfDiscardedResults);
 	printMeasurements(averageTimings, existingTimings);
@@ -98,9 +114,9 @@ async function buildAndGetTimings(config) {
 		config.output = config.output[0];
 	}
 	gc();
-	process.chdir(targetDir);
-	const bundle = await rollup.rollup(config);
-	process.chdir(initialDir);
+	chdir(targetDirectory);
+	const bundle = await rollup(config);
+	chdir(initialDirectory);
 	await bundle.generate(config.output);
 	return bundle.getTimings();
 }
@@ -108,12 +124,12 @@ async function buildAndGetTimings(config) {
 function printMeasurements(average, existingAverage, filter = /.*/) {
 	const printedLabels = Object.keys(average).filter(label => filter.test(label));
 	console.info('');
-	printedLabels.forEach(label => {
-		let color = text => text;
+	for (const label of printedLabels) {
+		let color = identity;
 		if (label[0] === '#') {
-			color = colorette.bold;
+			color = bold;
 			if (label[1] !== '#') {
-				color = colorette.underline;
+				color = underline;
 			}
 		}
 		console.info(
@@ -127,7 +143,7 @@ function printMeasurements(average, existingAverage, filter = /.*/) {
 				)}`
 			)
 		);
-	});
+	}
 	return printedLabels.length + 2;
 }
 
@@ -139,12 +155,10 @@ function getExistingTimings() {
 	try {
 		const timings = JSON.parse(readFileSync(perfFile, 'utf8'));
 		console.info(
-			colorette.bold(
-				`Comparing with ${colorette.cyan(perfFile)}. Delete this file to create a new base line.`
-			)
+			bold(`Comparing with ${cyan(perfFile)}. Delete this file to create a new base line.`)
 		);
 		return timings;
-	} catch (e) {
+	} catch {
 		return {};
 	}
 }
@@ -152,24 +166,15 @@ function getExistingTimings() {
 function persistTimings(timings) {
 	try {
 		writeFileSync(perfFile, JSON.stringify(timings, null, 2), 'utf8');
-		console.info(
-			colorette.bold(
-				`Saving performance information to new reference file ${colorette.cyan(perfFile)}.`
-			)
-		);
-	} catch (e) {
-		console.error(
-			colorette.bold(`Could not persist performance information in ${colorette.cyan(perfFile)}.`)
-		);
-		process.exit(1);
+		console.info(bold(`Saving performance information to new reference file ${cyan(perfFile)}.`));
+	} catch {
+		console.error(bold(`Could not persist performance information in ${cyan(perfFile)}.`));
+		exit(1);
 	}
 }
 
-const MIN_ABSOLUTE_TIME_DEVIATION = 10;
-const RELATIVE_DEVIATION_FOR_COLORING = 5;
-
 function getFormattedTime(currentTime, persistedTime = currentTime) {
-	let color = text => text,
+	let color = identity,
 		formattedTime = `${currentTime.toFixed(0)}ms`;
 	const absoluteDeviation = Math.abs(currentTime - persistedTime);
 	if (absoluteDeviation > MIN_ABSOLUTE_TIME_DEVIATION) {
@@ -179,25 +184,23 @@ function getFormattedTime(currentTime, persistedTime = currentTime) {
 			0
 		)}ms, ${sign}${relativeDeviation.toFixed(1)}%)`;
 		if (relativeDeviation > RELATIVE_DEVIATION_FOR_COLORING) {
-			color = currentTime >= persistedTime ? colorette.red : colorette.green;
+			color = currentTime >= persistedTime ? red : green;
 		}
 	}
 	return color(formattedTime);
 }
 
 function getFormattedMemory(currentMemory, persistedMemory = currentMemory) {
-	let color = text => text,
+	let color = identity,
 		formattedMemory = prettyBytes(currentMemory);
 	const absoluteDeviation = Math.abs(currentMemory - persistedMemory);
 	const sign = currentMemory >= persistedMemory ? '+' : '-';
 	const relativeDeviation = 100 * (absoluteDeviation / persistedMemory);
 	if (relativeDeviation > RELATIVE_DEVIATION_FOR_COLORING) {
 		formattedMemory += ` (${sign}${relativeDeviation.toFixed(0)}%)`;
-		color = currentMemory >= persistedMemory ? colorette.red : colorette.green;
+		color = currentMemory >= persistedMemory ? red : green;
 	}
 	return color(formattedMemory);
 }
 
-loadPerfConfig().then(async config =>
-	calculatePrintAndPersistTimings(config, await getExistingTimings())
-);
+const identity = x => x;
