@@ -16,10 +16,14 @@ import type {
 import { PluginDriver } from './utils/PluginDriver';
 import Queue from './utils/Queue';
 import { BuildPhase } from './utils/buildPhase';
-import { errImplicitDependantIsNotIncluded, error } from './utils/error';
+import {
+	error,
+	errorCircularDependency,
+	errorImplicitDependantIsNotIncluded,
+	errorMissingExport
+} from './utils/error';
 import { analyseModuleExecution } from './utils/executionOrder';
 import { addAnnotations } from './utils/pureComments';
-import relativeId from './utils/relativeId';
 import { timeEnd, timeStart } from './utils/timers';
 import { markModuleAndImpureDependenciesAsExecuted } from './utils/traverseStaticDependencies';
 
@@ -80,14 +84,14 @@ export default class Graph {
 
 		if (watcher) {
 			this.watchMode = true;
-			const handleChange = (...args: Parameters<WatchChangeHook>) =>
-				this.pluginDriver.hookParallel('watchChange', args);
+			const handleChange = (...parameters: Parameters<WatchChangeHook>) =>
+				this.pluginDriver.hookParallel('watchChange', parameters);
 			const handleClose = () => this.pluginDriver.hookParallel('closeWatcher', []);
-			watcher.onCurrentAwaited('change', handleChange);
-			watcher.onCurrentAwaited('close', handleClose);
+			watcher.onCurrentRun('change', handleChange);
+			watcher.onCurrentRun('close', handleClose);
 		}
 		this.pluginDriver = new PluginDriver(this, options, options.plugins, this.pluginCache);
-		this.acornParser = acorn.Parser.extend(...(options.acornInjectPlugins as any));
+		this.acornParser = acorn.Parser.extend(...(options.acornInjectPlugins as any[]));
 		this.moduleLoader = new ModuleLoader(this, this.modulesById, this.options, this.pluginDriver);
 		this.fileOperationQueue = new Queue(options.maxParallelFileOps);
 	}
@@ -97,10 +101,10 @@ export default class Graph {
 		await this.generateModuleGraph();
 		timeEnd('generate module graph', 2);
 
-		timeStart('sort modules', 2);
+		timeStart('sort and bind modules', 2);
 		this.phase = BuildPhase.ANALYSE;
 		this.sortModules();
-		timeEnd('sort modules', 2);
+		timeEnd('sort and bind modules', 2);
 
 		timeStart('mark included statements', 2);
 		this.includeStatements();
@@ -113,14 +117,13 @@ export default class Graph {
 		const onCommentOrig = options.onComment;
 		const comments: acorn.Comment[] = [];
 
-		if (onCommentOrig && typeof onCommentOrig == 'function') {
-			options.onComment = (block, text, start, end, ...args) => {
-				comments.push({ end, start, type: block ? 'Block' : 'Line', value: text });
-				return onCommentOrig.call(options, block, text, start, end, ...args);
-			};
-		} else {
-			options.onComment = comments;
-		}
+		options.onComment =
+			onCommentOrig && typeof onCommentOrig == 'function'
+				? (block, text, start, end, ...parameters) => {
+						comments.push({ end, start, type: block ? 'Block' : 'Line', value: text });
+						return onCommentOrig.call(options, block, text, start, end, ...parameters);
+				  }
+				: comments;
 
 		const ast = this.acornParser.parse(code, {
 			...(this.options.acorn as unknown as acorn.Options),
@@ -214,7 +217,7 @@ export default class Graph {
 		for (const module of this.implicitEntryModules) {
 			for (const dependant of module.implicitlyLoadedAfter) {
 				if (!(dependant.info.isEntry || dependant.isIncluded())) {
-					error(errImplicitDependantIsNotIncluded(dependant));
+					error(errorImplicitDependantIsNotIncluded(dependant));
 				}
 			}
 		}
@@ -223,12 +226,7 @@ export default class Graph {
 	private sortModules(): void {
 		const { orderedModules, cyclePaths } = analyseModuleExecution(this.entryModules);
 		for (const cyclePath of cyclePaths) {
-			this.options.onwarn({
-				code: 'CIRCULAR_DEPENDENCY',
-				cycle: cyclePath,
-				importer: cyclePath[0],
-				message: `Circular dependency: ${cyclePath.join(' -> ')}`
-			});
+			this.options.onwarn(errorCircularDependency(cyclePath));
 		}
 		this.modules = orderedModules;
 		for (const module of this.modules) {
@@ -245,14 +243,7 @@ export default class Graph {
 					!importDescription.module.getVariableForExportName(importDescription.name)[0]
 				) {
 					module.warn(
-						{
-							code: 'NON_EXISTENT_EXPORT',
-							message: `Non-existent export '${
-								importDescription.name
-							}' is imported from ${relativeId(importDescription.module.id)}`,
-							name: importDescription.name,
-							source: importDescription.module.id
-						},
+						errorMissingExport(importDescription.name, module.id, importDescription.module.id),
 						importDescription.start
 					);
 				}
