@@ -1,7 +1,10 @@
 import type { DeoptimizableEntity } from '../DeoptimizableEntity';
 import type { HasEffectsContext } from '../ExecutionContext';
-import type { NodeInteractionCalled, NodeInteractionWithThisArg } from '../NodeInteractions';
-import { NodeInteraction } from '../NodeInteractions';
+import type {
+	NodeInteraction,
+	NodeInteractionCalled,
+	NodeInteractionWithThisArgument
+} from '../NodeInteractions';
 import {
 	type ObjectPath,
 	type PathTracker,
@@ -26,7 +29,7 @@ export default class ArrayExpression extends NodeBase {
 	}
 
 	deoptimizeThisOnInteractionAtPath(
-		interaction: NodeInteractionWithThisArg,
+		interaction: NodeInteractionWithThisArgument,
 		path: ObjectPath,
 		recursionTracker: PathTracker
 	): void {
@@ -68,11 +71,9 @@ export default class ArrayExpression extends NodeBase {
 		let hasSpread = false;
 		for (let index = 0; index < this.elements.length; index++) {
 			const element = this.elements[index];
-			if (element) {
-				if (hasSpread || element instanceof SpreadElement) {
-					hasSpread = true;
-					element.deoptimizePath(UNKNOWN_PATH);
-				}
+			if (element && (hasSpread || element instanceof SpreadElement)) {
+				hasSpread = true;
+				element.deoptimizePath(UNKNOWN_PATH);
 			}
 		}
 		this.context.requestTreeshakingPass();
