@@ -1,21 +1,19 @@
 import type MagicString from 'magic-string';
+import { errorCannotCallNamespace } from '../../utils/error';
 import { type RenderOptions } from '../../utils/renderHelpers';
-import type { HasEffectsContext } from '../ExecutionContext';
-import { InclusionContext } from '../ExecutionContext';
-import { INTERACTION_CALLED, NodeInteractionWithThisArg } from '../NodeInteractions';
-import {
-	EMPTY_PATH,
-	PathTracker,
-	SHARED_RECURSION_TRACKER,
-	UNKNOWN_PATH
-} from '../utils/PathTracker';
-import Identifier from './Identifier';
+import type { HasEffectsContext, InclusionContext } from '../ExecutionContext';
+import type { NodeInteractionWithThisArgument } from '../NodeInteractions';
+import { INTERACTION_CALLED } from '../NodeInteractions';
+import type { PathTracker } from '../utils/PathTracker';
+import { EMPTY_PATH, SHARED_RECURSION_TRACKER, UNKNOWN_PATH } from '../utils/PathTracker';
+import type Identifier from './Identifier';
 import MemberExpression from './MemberExpression';
 import * as NodeType from './NodeType';
 import type TemplateLiteral from './TemplateLiteral';
 import CallExpressionBase from './shared/CallExpressionBase';
-import { ExpressionEntity, UNKNOWN_EXPRESSION } from './shared/Expression';
-import { type ExpressionNode, IncludeChildren } from './shared/Node';
+import type { ExpressionEntity } from './shared/Expression';
+import { UNKNOWN_EXPRESSION } from './shared/Expression';
+import type { ExpressionNode, IncludeChildren } from './shared/Node';
 
 export default class TaggedTemplateExpression extends CallExpressionBase {
 	declare quasi: TemplateLiteral;
@@ -29,13 +27,7 @@ export default class TaggedTemplateExpression extends CallExpressionBase {
 			const variable = this.scope.findVariable(name);
 
 			if (variable.isNamespace) {
-				this.context.warn(
-					{
-						code: 'CANNOT_CALL_NAMESPACE',
-						message: `Cannot call a namespace ('${name}')`
-					},
-					this.start
-				);
+				this.context.warn(errorCannotCallNamespace(name), this.start);
 			}
 		}
 	}
@@ -88,7 +80,7 @@ export default class TaggedTemplateExpression extends CallExpressionBase {
 		this.deoptimized = true;
 		if (this.interaction.thisArg) {
 			this.tag.deoptimizeThisOnInteractionAtPath(
-				this.interaction as NodeInteractionWithThisArg,
+				this.interaction as NodeInteractionWithThisArgument,
 				EMPTY_PATH,
 				SHARED_RECURSION_TRACKER
 			);
