@@ -1,20 +1,25 @@
-import { DeoptimizableEntity } from '../../DeoptimizableEntity';
-import { WritableEntity } from '../../Entity';
-import { HasEffectsContext, InclusionContext } from '../../ExecutionContext';
-import {
+import type { DeoptimizableEntity } from '../../DeoptimizableEntity';
+import type { WritableEntity } from '../../Entity';
+import type { HasEffectsContext, InclusionContext } from '../../ExecutionContext';
+import type {
 	NodeInteraction,
 	NodeInteractionCalled,
-	NodeInteractionWithThisArg
+	NodeInteractionWithThisArgument
 } from '../../NodeInteractions';
-import { ObjectPath, PathTracker, UNKNOWN_PATH } from '../../utils/PathTracker';
-import { LiteralValue } from '../Literal';
-import SpreadElement from '../SpreadElement';
-import { IncludeChildren } from './Node';
+import type { ObjectPath, PathTracker, SymbolToStringTag } from '../../utils/PathTracker';
+import { UNKNOWN_PATH } from '../../utils/PathTracker';
+import type { LiteralValue } from '../Literal';
+import type SpreadElement from '../SpreadElement';
+import type { IncludeChildren } from './Node';
 
 export const UnknownValue = Symbol('Unknown Value');
 export const UnknownTruthyValue = Symbol('Unknown Truthy Value');
 
-export type LiteralValueOrUnknown = LiteralValue | typeof UnknownValue | typeof UnknownTruthyValue;
+export type LiteralValueOrUnknown =
+	| LiteralValue
+	| typeof UnknownValue
+	| typeof UnknownTruthyValue
+	| typeof SymbolToStringTag;
 
 export interface InclusionOptions {
 	/**
@@ -29,7 +34,7 @@ export class ExpressionEntity implements WritableEntity {
 	deoptimizePath(_path: ObjectPath): void {}
 
 	deoptimizeThisOnInteractionAtPath(
-		{ thisArg }: NodeInteractionWithThisArg,
+		{ thisArg }: NodeInteractionWithThisArgument,
 		_path: ObjectPath,
 		_recursionTracker: PathTracker
 	): void {
@@ -39,7 +44,7 @@ export class ExpressionEntity implements WritableEntity {
 	/**
 	 * If possible it returns a stringifyable literal value for this node that can be used
 	 * for inlining or comparing values.
-	 * Otherwise it should return UnknownValue.
+	 * Otherwise, it should return UnknownValue.
 	 */
 	getLiteralValueAtPath(
 		_path: ObjectPath,
@@ -76,10 +81,10 @@ export class ExpressionEntity implements WritableEntity {
 
 	includeCallArguments(
 		context: InclusionContext,
-		args: readonly (ExpressionEntity | SpreadElement)[]
+		parameters: readonly (ExpressionEntity | SpreadElement)[]
 	): void {
-		for (const arg of args) {
-			arg.include(context, false);
+		for (const argument of parameters) {
+			argument.include(context, false);
 		}
 	}
 
