@@ -1,12 +1,17 @@
-import Module, { AstContext } from '../../Module';
+import type { AstContext } from '../../Module';
+import type Module from '../../Module';
 import type { DeoptimizableEntity } from '../DeoptimizableEntity';
-import { createInclusionContext, HasEffectsContext, InclusionContext } from '../ExecutionContext';
-import type { NodeInteractionCalled, NodeInteractionWithThisArg } from '../NodeInteractions';
+import type { HasEffectsContext, InclusionContext } from '../ExecutionContext';
+import { createInclusionContext } from '../ExecutionContext';
+import type {
+	NodeInteraction,
+	NodeInteractionCalled,
+	NodeInteractionWithThisArgument
+} from '../NodeInteractions';
 import {
 	INTERACTION_ACCESSED,
 	INTERACTION_ASSIGNED,
-	INTERACTION_CALLED,
-	NodeInteraction
+	INTERACTION_CALLED
 } from '../NodeInteractions';
 import type ExportDefaultDeclaration from '../nodes/ExportDefaultDeclaration';
 import type Identifier from '../nodes/Identifier';
@@ -87,7 +92,7 @@ export default class LocalVariable extends Variable {
 	}
 
 	deoptimizeThisOnInteractionAtPath(
-		interaction: NodeInteractionWithThisArg,
+		interaction: NodeInteractionWithThisArgument,
 		path: ObjectPath,
 		recursionTracker: PathTracker
 	): void {
@@ -152,25 +157,28 @@ export default class LocalVariable extends Variable {
 		context: HasEffectsContext
 	): boolean {
 		switch (interaction.type) {
-			case INTERACTION_ACCESSED:
+			case INTERACTION_ACCESSED: {
 				if (this.isReassigned) return true;
 				return (this.init &&
 					!context.accessed.trackEntityAtPathAndGetIfTracked(path, this) &&
 					this.init.hasEffectsOnInteractionAtPath(path, interaction, context))!;
-			case INTERACTION_ASSIGNED:
+			}
+			case INTERACTION_ASSIGNED: {
 				if (this.included) return true;
 				if (path.length === 0) return false;
 				if (this.isReassigned) return true;
 				return (this.init &&
 					!context.assigned.trackEntityAtPathAndGetIfTracked(path, this) &&
 					this.init.hasEffectsOnInteractionAtPath(path, interaction, context))!;
-			case INTERACTION_CALLED:
+			}
+			case INTERACTION_CALLED: {
 				if (this.isReassigned) return true;
 				return (this.init &&
 					!(
 						interaction.withNew ? context.instantiated : context.called
 					).trackEntityAtPathAndGetIfTracked(path, interaction.args, this) &&
 					this.init.hasEffectsOnInteractionAtPath(path, interaction, context))!;
+			}
 		}
 	}
 
@@ -194,15 +202,15 @@ export default class LocalVariable extends Variable {
 
 	includeCallArguments(
 		context: InclusionContext,
-		args: readonly (ExpressionEntity | SpreadElement)[]
+		parameters: readonly (ExpressionEntity | SpreadElement)[]
 	): void {
 		if (this.isReassigned || (this.init && context.includedCallArguments.has(this.init))) {
-			for (const arg of args) {
-				arg.include(context, false);
+			for (const argument of parameters) {
+				argument.include(context, false);
 			}
 		} else if (this.init) {
 			context.includedCallArguments.add(this.init);
-			this.init.includeCallArguments(context, args);
+			this.init.includeCallArguments(context, parameters);
 			context.includedCallArguments.delete(this.init);
 		}
 	}
