@@ -3,27 +3,27 @@ import type Graph from '../Graph';
 import type Module from '../Module';
 import type {
 	EmittedChunk,
-	FilePlaceholder,
 	NormalizedInputOptions,
 	NormalizedOutputOptions,
-	OutputBundleWithPlaceholders,
 	WarningHandler
 } from '../rollup/types';
 import { BuildPhase } from './buildPhase';
 import { createHash } from './crypto';
 import {
-	errAssetNotFinalisedForFileName,
-	errAssetReferenceIdNotFoundForSetSource,
-	errAssetSourceAlreadySet,
-	errChunkNotGeneratedForFileName,
-	errFailedValidation,
-	errFileNameConflict,
-	errFileReferenceIdNotFoundForFilename,
-	errInvalidRollupPhaseForChunkEmission,
-	errNoAssetSourceSet,
 	error,
-	warnDeprecation
+	errorAssetNotFinalisedForFileName,
+	errorAssetReferenceIdNotFoundForSetSource,
+	errorAssetSourceAlreadySet,
+	errorChunkNotGeneratedForFileName,
+	errorFailedValidation,
+	errorFileNameConflict,
+	errorFileReferenceIdNotFoundForFilename,
+	errorInvalidRollupPhaseForChunkEmission,
+	errorNoAssetSourceSet
 } from './error';
+import { defaultHashSize } from './hashPlaceholders';
+import type { OutputBundleWithPlaceholders } from './outputBundle';
+import { FILE_PLACEHOLDER, lowercaseBundleKeys } from './outputBundle';
 import { extname } from './path';
 import { isPathFragment } from './relativeId';
 import { makeUnique, renderNamePattern } from './renderNamePattern';
@@ -42,17 +42,15 @@ function generateAssetFileName(
 				: outputOptions.assetFileNames,
 			'output.assetFileNames',
 			{
-				ext: () => extname(emittedName).substring(1),
+				ext: () => extname(emittedName).slice(1),
 				extname: () => extname(emittedName),
-				hash() {
-					return createHash()
-						.update(emittedName)
-						.update(':')
+				hash: size =>
+					createHash()
 						.update(source)
 						.digest('hex')
-						.substring(0, 8);
-				},
-				name: () => emittedName.substring(0, emittedName.length - extname(emittedName).length)
+						.slice(0, Math.max(0, size || defaultHashSize)),
+				name: () =>
+					emittedName.slice(0, Math.max(0, emittedName.length - extname(emittedName).length))
 			}
 		),
 		bundle
@@ -61,13 +59,14 @@ function generateAssetFileName(
 
 function reserveFileNameInBundle(
 	fileName: string,
-	bundle: OutputBundleWithPlaceholders,
+	{ bundle }: FileEmitterOutput,
 	warn: WarningHandler
 ) {
-	if (fileName in bundle) {
-		warn(errFileNameConflict(fileName));
+	if (bundle[lowercaseBundleKeys].has(fileName.toLowerCase())) {
+		warn(errorFileNameConflict(fileName));
+	} else {
+		bundle[fileName] = FILE_PLACEHOLDER;
 	}
-	bundle[fileName] = FILE_PLACEHOLDER;
 }
 
 interface ConsumedChunk {
@@ -93,10 +92,6 @@ interface EmittedFile {
 
 type ConsumedFile = ConsumedChunk | ConsumedAsset;
 
-export const FILE_PLACEHOLDER: FilePlaceholder = {
-	type: 'placeholder'
-};
-
 function hasValidType(
 	emittedFile: unknown
 ): emittedFile is { [key: string]: unknown; type: 'asset' | 'chunk' } {
@@ -123,7 +118,7 @@ function getValidSource(
 	if (!(typeof source === 'string' || source instanceof Uint8Array)) {
 		const assetName = emittedFile.fileName || emittedFile.name || fileReferenceId;
 		return error(
-			errFailedValidation(
+			errorFailedValidation(
 				`Could not set source for ${
 					typeof assetName === 'string' ? `asset "${assetName}"` : 'unnamed asset'
 				}, asset source needs to be a string, Uint8Array or Buffer.`
@@ -135,7 +130,7 @@ function getValidSource(
 
 function getAssetFileName(file: ConsumedAsset, referenceId: string): string {
 	if (typeof file.fileName !== 'string') {
-		return error(errAssetNotFinalisedForFileName(file.name || referenceId));
+		return error(errorAssetNotFinalisedForFileName(file.name || referenceId));
 	}
 	return file.fileName;
 }
@@ -144,16 +139,27 @@ function getChunkFileName(
 	file: ConsumedChunk,
 	facadeChunkByModule: ReadonlyMap<Module, Chunk> | null
 ): string {
-	const fileName = file.fileName || (file.module && facadeChunkByModule?.get(file.module)?.id);
-	if (!fileName) return error(errChunkNotGeneratedForFileName(file.fileName || file.name));
-	return fileName;
+	if (file.fileName) {
+		return file.fileName;
+	}
+	if (facadeChunkByModule) {
+		const chunk = facadeChunkByModule.get(file.module!)!;
+		return chunk.id || chunk.getFileName();
+	}
+	return error(errorChunkNotGeneratedForFileName(file.fileName || file.name));
+}
+
+interface FileEmitterOutput {
+	bundle: OutputBundleWithPlaceholders;
+	fileNamesBySource: Map<string, string>;
+	outputOptions: NormalizedOutputOptions;
 }
 
 export class FileEmitter {
-	private bundle: OutputBundleWithPlaceholders | null = null;
 	private facadeChunkByModule: ReadonlyMap<Module, Chunk> | null = null;
 	private readonly filesByReferenceId: Map<string, ConsumedFile>;
-	private outputOptions: NormalizedOutputOptions | null = null;
+	private nextIdBase = 1;
+	private output: FileEmitterOutput | null = null;
 
 	constructor(
 		private readonly graph: Graph,
@@ -165,17 +171,10 @@ export class FileEmitter {
 			: new Map();
 	}
 
-	public assertAssetsFinalized = (): void => {
-		for (const [referenceId, emittedFile] of this.filesByReferenceId) {
-			if (emittedFile.type === 'asset' && typeof emittedFile.fileName !== 'string')
-				return error(errNoAssetSourceSet(emittedFile.name || referenceId));
-		}
-	};
-
 	public emitFile = (emittedFile: unknown): string => {
 		if (!hasValidType(emittedFile)) {
 			return error(
-				errFailedValidation(
+				errorFailedValidation(
 					`Emitted files must be of type "asset" or "chunk", received "${
 						emittedFile && (emittedFile as any).type
 					}".`
@@ -184,7 +183,7 @@ export class FileEmitter {
 		}
 		if (!hasValidName(emittedFile)) {
 			return error(
-				errFailedValidation(
+				errorFailedValidation(
 					`The "fileName" or "name" properties of emitted files must be strings that are neither absolute nor relative paths, received "${
 						emittedFile.fileName || emittedFile.name
 					}".`
@@ -197,9 +196,16 @@ export class FileEmitter {
 		return this.emitAsset(emittedFile);
 	};
 
+	public finaliseAssets = (): void => {
+		for (const [referenceId, emittedFile] of this.filesByReferenceId) {
+			if (emittedFile.type === 'asset' && typeof emittedFile.fileName !== 'string')
+				return error(errorNoAssetSourceSet(emittedFile.name || referenceId));
+		}
+	};
+
 	public getFileName = (fileReferenceId: string): string => {
 		const emittedFile = this.filesByReferenceId.get(fileReferenceId);
-		if (!emittedFile) return error(errFileReferenceIdNotFoundForFilename(fileReferenceId));
+		if (!emittedFile) return error(errorFileReferenceIdNotFoundForFilename(fileReferenceId));
 		if (emittedFile.type === 'chunk') {
 			return getChunkFileName(emittedFile, this.facadeChunkByModule);
 		}
@@ -208,41 +214,46 @@ export class FileEmitter {
 
 	public setAssetSource = (referenceId: string, requestedSource: unknown): void => {
 		const consumedFile = this.filesByReferenceId.get(referenceId);
-		if (!consumedFile) return error(errAssetReferenceIdNotFoundForSetSource(referenceId));
+		if (!consumedFile) return error(errorAssetReferenceIdNotFoundForSetSource(referenceId));
 		if (consumedFile.type !== 'asset') {
 			return error(
-				errFailedValidation(
+				errorFailedValidation(
 					`Asset sources can only be set for emitted assets but "${referenceId}" is an emitted chunk.`
 				)
 			);
 		}
 		if (consumedFile.source !== undefined) {
-			return error(errAssetSourceAlreadySet(consumedFile.name || referenceId));
+			return error(errorAssetSourceAlreadySet(consumedFile.name || referenceId));
 		}
 		const source = getValidSource(requestedSource, consumedFile, referenceId);
-		if (this.bundle) {
-			this.finalizeAsset(consumedFile, source, referenceId, this.bundle);
+		if (this.output) {
+			this.finalizeAsset(consumedFile, source, referenceId, this.output);
 		} else {
 			consumedFile.source = source;
 		}
 	};
 
-	public setOutputBundle = (
-		outputBundle: OutputBundleWithPlaceholders,
-		outputOptions: NormalizedOutputOptions,
-		facadeChunkByModule: ReadonlyMap<Module, Chunk>
-	): void => {
-		this.outputOptions = outputOptions;
-		this.bundle = outputBundle;
+	public setChunkInformation = (facadeChunkByModule: ReadonlyMap<Module, Chunk>): void => {
 		this.facadeChunkByModule = facadeChunkByModule;
+	};
+
+	public setOutputBundle = (
+		bundle: OutputBundleWithPlaceholders,
+		outputOptions: NormalizedOutputOptions
+	): void => {
+		const fileNamesBySource = new Map();
+		const output = (this.output = { bundle, fileNamesBySource, outputOptions });
 		for (const emittedFile of this.filesByReferenceId.values()) {
 			if (emittedFile.fileName) {
-				reserveFileNameInBundle(emittedFile.fileName, this.bundle, this.options.onwarn);
+				reserveFileNameInBundle(emittedFile.fileName, output, this.options.onwarn);
+				if (emittedFile.type === 'asset' && typeof emittedFile.source === 'string') {
+					fileNamesBySource.set(emittedFile.source, emittedFile.fileName);
+				}
 			}
 		}
 		for (const [referenceId, consumedFile] of this.filesByReferenceId) {
 			if (consumedFile.type === 'asset' && consumedFile.source !== undefined) {
-				this.finalizeAsset(consumedFile, consumedFile.source, referenceId, this.bundle);
+				this.finalizeAsset(consumedFile, consumedFile.source, referenceId, output);
 			}
 		}
 	};
@@ -254,7 +265,7 @@ export class FileEmitter {
 			referenceId = createHash()
 				.update(referenceId || idBase)
 				.digest('hex')
-				.substring(0, 8);
+				.slice(0, 8);
 		} while (this.filesByReferenceId.has(referenceId));
 
 		this.filesByReferenceId.set(referenceId, file);
@@ -274,14 +285,14 @@ export class FileEmitter {
 		};
 		const referenceId = this.assignReferenceId(
 			consumedAsset,
-			emittedAsset.fileName || emittedAsset.name || emittedAsset.type
+			emittedAsset.fileName || emittedAsset.name || String(this.nextIdBase++)
 		);
-		if (this.bundle) {
+		if (this.output) {
 			if (emittedAsset.fileName) {
-				reserveFileNameInBundle(emittedAsset.fileName, this.bundle, this.options.onwarn);
+				reserveFileNameInBundle(emittedAsset.fileName, this.output, this.options.onwarn);
 			}
 			if (source !== undefined) {
-				this.finalizeAsset(consumedAsset, source, referenceId, this.bundle);
+				this.finalizeAsset(consumedAsset, source, referenceId, this.output);
 			}
 		}
 		return referenceId;
@@ -289,11 +300,11 @@ export class FileEmitter {
 
 	private emitChunk(emittedChunk: EmittedFile): string {
 		if (this.graph.phase > BuildPhase.LOAD_AND_PARSE) {
-			return error(errInvalidRollupPhaseForChunkEmission());
+			return error(errorInvalidRollupPhaseForChunkEmission());
 		}
 		if (typeof emittedChunk.id !== 'string') {
 			return error(
-				errFailedValidation(
+				errorFailedValidation(
 					`Emitted chunks need to have a valid string id, received "${emittedChunk.id}"`
 				)
 			);
@@ -319,65 +330,24 @@ export class FileEmitter {
 		consumedFile: ConsumedFile,
 		source: string | Uint8Array,
 		referenceId: string,
-		bundle: OutputBundleWithPlaceholders
+		{ bundle, fileNamesBySource, outputOptions }: FileEmitterOutput
 	): void {
 		const fileName =
 			consumedFile.fileName ||
-			findExistingAssetFileNameWithSource(bundle, source) ||
-			generateAssetFileName(consumedFile.name, source, this.outputOptions!, bundle);
+			(typeof source === 'string' && fileNamesBySource.get(source)) ||
+			generateAssetFileName(consumedFile.name, source, outputOptions, bundle);
 
 		// We must not modify the original assets to avoid interaction between outputs
 		const assetWithFileName = { ...consumedFile, fileName, source };
 		this.filesByReferenceId.set(referenceId, assetWithFileName);
-		const { options } = this;
+		if (typeof source === 'string') {
+			fileNamesBySource.set(source, fileName);
+		}
 		bundle[fileName] = {
 			fileName,
-			get isAsset(): true {
-				warnDeprecation(
-					'Accessing "isAsset" on files in the bundle is deprecated, please use "type === \'asset\'" instead',
-					true,
-					options
-				);
-
-				return true;
-			},
 			name: consumedFile.name,
 			source,
 			type: 'asset'
 		};
 	}
 }
-
-function findExistingAssetFileNameWithSource(
-	bundle: OutputBundleWithPlaceholders,
-	source: string | Uint8Array
-): string | null {
-	for (const [fileName, outputFile] of Object.entries(bundle)) {
-		if (outputFile.type === 'asset' && areSourcesEqual(source, outputFile.source)) return fileName;
-	}
-	return null;
-}
-
-function areSourcesEqual(
-	sourceA: string | Uint8Array | Buffer,
-	sourceB: string | Uint8Array | Buffer
-): boolean {
-	if (typeof sourceA === 'string') {
-		return sourceA === sourceB;
-	}
-	if (typeof sourceB === 'string') {
-		return false;
-	}
-	if ('equals' in sourceA) {
-		return sourceA.equals(sourceB);
-	}
-	if (sourceA.length !== sourceB.length) {
-		return false;
-	}
-	for (let index = 0; index < sourceA.length; index++) {
-		if (sourceA[index] !== sourceB[index]) {
-			return false;
-		}
-	}
-	return true;
-}
