@@ -4,8 +4,8 @@ import type {
 	MergedRollupOptions,
 	OutputOptions,
 	RollupCache,
-	WarningHandler,
-	WarningHandlerWithDefault
+	RollupOptions,
+	WarningHandler
 } from '../../rollup/types';
 import { ensureArray } from '../ensureArray';
 import type { CommandConfigObject } from './normalizeInputOptions';
@@ -13,6 +13,7 @@ import {
 	defaultOnWarn,
 	generatedCodePresets,
 	type GenericConfigObject,
+	normalizePluginOption,
 	objectifyOption,
 	objectifyOptionWithPresets,
 	treeshakePresets,
@@ -35,28 +36,34 @@ export const commandAliases: { [key: string]: string } = {
 	w: 'watch'
 };
 
-export function mergeOptions(
-	config: GenericConfigObject,
-	rawCommandOptions: GenericConfigObject = { external: [], globals: undefined },
+const EMPTY_COMMAND_OPTIONS = { external: [], globals: undefined };
+
+export async function mergeOptions(
+	config: RollupOptions,
+	rawCommandOptions: GenericConfigObject = EMPTY_COMMAND_OPTIONS,
 	defaultOnWarnHandler: WarningHandler = defaultOnWarn
-): MergedRollupOptions {
+): Promise<MergedRollupOptions> {
 	const command = getCommandOptions(rawCommandOptions);
-	const inputOptions = mergeInputOptions(config, command, defaultOnWarnHandler);
+	const inputOptions = await mergeInputOptions(config, command, defaultOnWarnHandler);
 	const warn = inputOptions.onwarn as WarningHandler;
 	if (command.output) {
 		Object.assign(command, command.output);
 	}
-	const outputOptionsArray = ensureArray(config.output) as GenericConfigObject[];
+	const outputOptionsArray = ensureArray(config.output);
 	if (outputOptionsArray.length === 0) outputOptionsArray.push({});
-	const outputOptions = outputOptionsArray.map(singleOutputOptions =>
-		mergeOutputOptions(singleOutputOptions, command, warn)
+	const outputOptions = await Promise.all(
+		outputOptionsArray.map(singleOutputOptions =>
+			mergeOutputOptions(singleOutputOptions, command, warn)
+		)
 	);
 
 	warnUnknownOptions(
 		command,
-		Object.keys(inputOptions).concat(
-			Object.keys(outputOptions[0]).filter(option => option !== 'sourcemapPathTransform'),
-			Object.keys(commandAliases),
+		[
+			...Object.keys(inputOptions),
+			...Object.keys(outputOptions[0]).filter(option => option !== 'sourcemapPathTransform'),
+			...Object.keys(commandAliases),
+			'bundleConfigAsCjs',
 			'config',
 			'environment',
 			'plugin',
@@ -65,7 +72,7 @@ export function mergeOptions(
 			'stdin',
 			'waitForBundleInput',
 			'configPlugin'
-		),
+		],
 		'CLI flags',
 		warn,
 		/^_$|output$|config/
@@ -100,12 +107,12 @@ type CompleteInputOptions<U extends keyof InputOptions> = {
 	[K in U]: InputOptions[K];
 };
 
-function mergeInputOptions(
-	config: GenericConfigObject,
+async function mergeInputOptions(
+	config: InputOptions,
 	overrides: CommandConfigObject,
 	defaultOnWarnHandler: WarningHandler
-): InputOptions {
-	const getOption = (name: string): any => overrides[name] ?? config[name];
+): Promise<InputOptions> {
+	const getOption = (name: keyof InputOptions): any => overrides[name] ?? config[name];
 	const inputOptions: CompleteInputOptions<keyof InputOptions> = {
 		acorn: getOption('acorn'),
 		acornInjectPlugins: config.acornInjectPlugins as
@@ -125,7 +132,7 @@ function mergeInputOptions(
 		moduleContext: getOption('moduleContext'),
 		onwarn: getOnWarn(config, defaultOnWarnHandler),
 		perf: getOption('perf'),
-		plugins: ensureArray(config.plugins) as Plugin[],
+		plugins: await normalizePluginOption(config.plugins),
 		preserveEntrySignatures: getOption('preserveEntrySignatures'),
 		preserveModules: getOption('preserveModules'),
 		preserveSymlinks: getOption('preserveSymlinks'),
@@ -150,29 +157,21 @@ function mergeInputOptions(
 	return inputOptions;
 }
 
-const getExternal = (
-	config: GenericConfigObject,
-	overrides: CommandConfigObject
-): ExternalOption => {
-	const configExternal = config.external as ExternalOption | undefined;
+const getExternal = (config: InputOptions, overrides: CommandConfigObject): ExternalOption => {
+	const configExternal = config.external;
 	return typeof configExternal === 'function'
 		? (source: string, importer: string | undefined, isResolved: boolean) =>
 				configExternal(source, importer, isResolved) || overrides.external.includes(source)
-		: ensureArray(configExternal).concat(overrides.external);
+		: [...ensureArray(configExternal), ...overrides.external];
 };
 
-const getOnWarn = (
-	config: GenericConfigObject,
-	defaultOnWarnHandler: WarningHandler
-): WarningHandler =>
-	config.onwarn
-		? warning => (config.onwarn as WarningHandlerWithDefault)(warning, defaultOnWarnHandler)
-		: defaultOnWarnHandler;
+const getOnWarn = (config: InputOptions, defaultOnWarnHandler: WarningHandler): WarningHandler =>
+	config.onwarn ? warning => config.onwarn!(warning, defaultOnWarnHandler) : defaultOnWarnHandler;
 
-const getObjectOption = (
-	config: GenericConfigObject,
-	overrides: GenericConfigObject,
-	name: string,
+const getObjectOption = <T extends object>(
+	config: T,
+	overrides: T,
+	name: keyof T,
 	objectifyValue = objectifyOption
 ) => {
 	const commandOption = normalizeObjectOptionValue(overrides[name], objectifyValue);
@@ -183,7 +182,7 @@ const getObjectOption = (
 	return configOption;
 };
 
-export const getWatch = (config: GenericConfigObject, overrides: GenericConfigObject) =>
+export const getWatch = (config: InputOptions, overrides: InputOptions) =>
 	config.watch !== false && getObjectOption(config, overrides, 'watch');
 
 export const isWatchEnabled = (optionValue: unknown): boolean => {
@@ -216,12 +215,12 @@ type CompleteOutputOptions<U extends keyof OutputOptions> = {
 	[K in U]: OutputOptions[K];
 };
 
-function mergeOutputOptions(
-	config: GenericConfigObject,
-	overrides: GenericConfigObject,
+async function mergeOutputOptions(
+	config: OutputOptions,
+	overrides: OutputOptions,
 	warn: WarningHandler
-): OutputOptions {
-	const getOption = (name: string): any => overrides[name] ?? config[name];
+): Promise<OutputOptions> {
+	const getOption = (name: keyof OutputOptions): any => overrides[name] ?? config[name];
 	const outputOptions: CompleteOutputOptions<keyof OutputOptions> = {
 		amd: getObjectOption(config, overrides, 'amd'),
 		assetFileNames: getOption('assetFileNames'),
@@ -230,10 +229,12 @@ function mergeOutputOptions(
 		compact: getOption('compact'),
 		dir: getOption('dir'),
 		dynamicImportFunction: getOption('dynamicImportFunction'),
+		dynamicImportInCjs: getOption('dynamicImportInCjs'),
 		entryFileNames: getOption('entryFileNames'),
 		esModule: getOption('esModule'),
 		exports: getOption('exports'),
 		extend: getOption('extend'),
+		externalImportAssertions: getOption('externalImportAssertions'),
 		externalLiveBindings: getOption('externalLiveBindings'),
 		file: getOption('file'),
 		footer: getOption('footer'),
@@ -258,7 +259,7 @@ function mergeOutputOptions(
 		noConflict: getOption('noConflict'),
 		outro: getOption('outro'),
 		paths: getOption('paths'),
-		plugins: ensureArray(config.plugins) as Plugin[],
+		plugins: await normalizePluginOption(config.plugins),
 		preferConst: getOption('preferConst'),
 		preserveModules: getOption('preserveModules'),
 		preserveModulesRoot: getOption('preserveModulesRoot'),
