@@ -1,27 +1,17 @@
-import type {
-	CustomPluginOptions,
-	Plugin,
-	PluginContext,
-	ResolvedId,
-	ResolveIdResult
-} from '../rollup/types';
+import type { ModuleLoaderResolveId } from '../ModuleLoader';
+import type { CustomPluginOptions, Plugin, PluginContext, ResolveIdResult } from '../rollup/types';
 import type { PluginDriver, ReplaceContext } from './PluginDriver';
-import { BLANK } from './blank';
+import { BLANK, EMPTY_OBJECT } from './blank';
 
 export function resolveIdViaPlugins(
 	source: string,
 	importer: string | undefined,
 	pluginDriver: PluginDriver,
-	moduleLoaderResolveId: (
-		source: string,
-		importer: string | undefined,
-		customOptions: CustomPluginOptions | undefined,
-		isEntry: boolean | undefined,
-		skip: readonly { importer: string | undefined; plugin: Plugin; source: string }[] | null
-	) => Promise<ResolvedId | null>,
+	moduleLoaderResolveId: ModuleLoaderResolveId,
 	skip: readonly { importer: string | undefined; plugin: Plugin; source: string }[] | null,
 	customOptions: CustomPluginOptions | undefined,
-	isEntry: boolean
+	isEntry: boolean,
+	assertions: Record<string, string>
 ): Promise<ResolveIdResult> {
 	let skipped: Set<Plugin> | null = null;
 	let replaceContext: ReplaceContext | null = null;
@@ -34,12 +24,13 @@ export function resolveIdViaPlugins(
 		}
 		replaceContext = (pluginContext, plugin): PluginContext => ({
 			...pluginContext,
-			resolve: (source, importer, { custom, isEntry, skipSelf } = BLANK) => {
+			resolve: (source, importer, { assertions, custom, isEntry, skipSelf } = BLANK) => {
 				return moduleLoaderResolveId(
 					source,
 					importer,
 					custom,
 					isEntry,
+					assertions || EMPTY_OBJECT,
 					skipSelf ? [...skip, { importer, plugin, source }] : skip
 				);
 			}
@@ -47,7 +38,7 @@ export function resolveIdViaPlugins(
 	}
 	return pluginDriver.hookFirst(
 		'resolveId',
-		[source, importer, { custom: customOptions, isEntry }],
+		[source, importer, { assertions, custom: customOptions, isEntry }],
 		replaceContext,
 		skipped
 	);
