@@ -1,5 +1,5 @@
-const assert = require('assert');
-const path = require('path');
+const assert = require('node:assert');
+const path = require('node:path');
 
 const ID_MAIN = path.join(__dirname, 'main.js');
 const ID_LIB = path.join(__dirname, 'lib.js');
@@ -25,6 +25,8 @@ module.exports = {
 			},
 			buildEnd() {
 				assert.deepStrictEqual(JSON.parse(JSON.stringify(this.getModuleInfo(ID_MAIN))), {
+					id: ID_MAIN,
+					assertions: {},
 					ast: {
 						type: 'Program',
 						start: 0,
@@ -75,11 +77,11 @@ module.exports = {
 					dynamicImporters: [],
 					hasDefaultExport: false,
 					moduleSideEffects: true,
-					id: ID_MAIN,
 					implicitlyLoadedAfterOneOf: [],
 					implicitlyLoadedBefore: [],
 					importedIdResolutions: [
 						{
+							assertions: {},
 							external: false,
 							id: ID_LIB,
 							meta: {},
@@ -96,6 +98,8 @@ module.exports = {
 					syntheticNamedExports: false
 				});
 				assert.deepStrictEqual(JSON.parse(JSON.stringify(this.getModuleInfo(ID_DEP))), {
+					id: ID_DEP,
+					assertions: {},
 					ast: {
 						type: 'Program',
 						start: 0,
@@ -146,11 +150,11 @@ module.exports = {
 					dynamicImporters: [],
 					hasDefaultExport: false,
 					moduleSideEffects: true,
-					id: ID_DEP,
 					implicitlyLoadedAfterOneOf: [],
 					implicitlyLoadedBefore: [],
 					importedIdResolutions: [
 						{
+							assertions: {},
 							external: false,
 							id: ID_LIB,
 							meta: {},
