@@ -22,7 +22,7 @@ const foo = 100;
 var ns = /*#__PURE__*/_mergeNamespaces({
   __proto__: null,
   foo: foo,
-  'default': d
+  default: d
 }, [d]);
 
 const stuff = 12;
