@@ -23,7 +23,7 @@ const foo = 'foo';
 
 var synthetic$1 = /*#__PURE__*/_mergeNamespaces({
 	__proto__: null,
-	'default': synthetic,
+	default: synthetic,
 	foo: foo
 }, [synthetic]);
 
@@ -33,7 +33,7 @@ var dep = 'not-overwritten';
 var dep$1 = /*#__PURE__*/_mergeNamespaces({
 	__proto__: null,
 	bar: bar,
-	'default': dep,
+	default: dep,
 	synthetic: synthetic$1,
 	foo: foo
 }, [synthetic]);
