@@ -1,4 +1,4 @@
-const path = require('path');
+const path = require('node:path');
 
 let resolveA;
 const waitForA = new Promise(resolve => (resolveA = resolve));
