@@ -1,5 +1,5 @@
-const assert = require('assert');
-const { promises: fs } = require('fs');
+const assert = require('node:assert');
+const { promises: fs } = require('node:fs');
 const { wait } = require('../../../../../utils');
 
 const fsReadFile = fs.readFile;
@@ -14,7 +14,7 @@ module.exports = {
 		plugins: [
 			{
 				load(id) {
-					return fs.readFile(id, 'utf-8');
+					return fs.readFile(id, 'utf8');
 				}
 			}
 		]
@@ -32,5 +32,12 @@ module.exports = {
 	after() {
 		fs.readFile = fsReadFile;
 		assert.strictEqual(maxReads, 3, 'Wrong number of parallel file reads: ' + maxReads);
-	}
+	},
+	warnings: [
+		{
+			code: 'DEPRECATED_FEATURE',
+			message:
+				'The "maxParallelFileReads" option is deprecated. Use the "maxParallelFileOps" option instead.'
+		}
+	]
 };
