@@ -1,4 +1,4 @@
-const assert = require('assert');
+const assert = require('node:assert');
 
 module.exports = {
 	description: 'avoids name conflicts with local variables named Symbol',
@@ -11,5 +11,12 @@ module.exports = {
 	exports(exports) {
 		assert.strictEqual(exports.Symbol, null);
 		assert.strictEqual(exports.toString(), '[object Module]');
-	}
+	},
+	warnings: [
+		{
+			code: 'DEPRECATED_FEATURE',
+			message:
+				'The "output.namespaceToStringTag" option is deprecated. Use the "output.generatedCode.symbols" option instead.'
+		}
+	]
 };
