@@ -1,5 +1,5 @@
-const assert = require('assert');
-const path = require('path');
+const assert = require('node:assert');
+const path = require('node:path');
 const { getObject } = require('../../../utils');
 
 function getId(name) {
@@ -21,6 +21,7 @@ module.exports = {
 					{
 						[getId('dynamic')]: {
 							id: getId('dynamic'),
+							assertions: {},
 							ast: {
 								type: 'Program',
 								start: 0,
@@ -81,6 +82,7 @@ module.exports = {
 							code: "export const promise = import('external');\nexport { default as internal } from './lib';\n",
 							dynamicallyImportedIdResolutions: [
 								{
+									assertions: {},
 									external: true,
 									id: 'external',
 									meta: {},
@@ -96,6 +98,7 @@ module.exports = {
 							implicitlyLoadedBefore: [],
 							importedIdResolutions: [
 								{
+									assertions: {},
 									external: false,
 									id: getId('lib'),
 									meta: {},
@@ -113,6 +116,7 @@ module.exports = {
 						},
 						[getId('lib')]: {
 							id: getId('lib'),
+							assertions: {},
 							ast: {
 								type: 'Program',
 								start: 0,
@@ -146,6 +150,7 @@ module.exports = {
 						},
 						[getId('main')]: {
 							id: getId('main'),
+							assertions: {},
 							ast: {
 								type: 'Program',
 								start: 0,
@@ -228,6 +233,7 @@ module.exports = {
 							code: "export const promise = import('./dynamic');\nexport { default as value } from './lib';\nexport { external } from 'external';\n",
 							dynamicallyImportedIdResolutions: [
 								{
+									assertions: {},
 									external: false,
 									id: getId('dynamic'),
 									meta: {},
@@ -243,6 +249,7 @@ module.exports = {
 							implicitlyLoadedBefore: [],
 							importedIdResolutions: [
 								{
+									assertions: {},
 									external: false,
 									id: getId('lib'),
 									meta: {},
@@ -250,6 +257,7 @@ module.exports = {
 									syntheticNamedExports: false
 								},
 								{
+									assertions: {},
 									external: true,
 									id: 'external',
 									meta: {},
@@ -267,6 +275,7 @@ module.exports = {
 						},
 						external: {
 							id: 'external',
+							assertions: {},
 							ast: null,
 							code: null,
 							dynamicallyImportedIdResolutions: [],
