@@ -1,4 +1,4 @@
-const assert = require('assert');
+const assert = require('node:assert');
 
 module.exports = {
 	description: 'allows to read and modify options in the options hook',
@@ -24,10 +24,12 @@ module.exports = {
 					assetFileNames: 'assets/[name]-[hash][extname]',
 					chunkFileNames: '[name]-[hash].js',
 					compact: false,
+					dynamicImportInCjs: true,
 					entryFileNames: '[name].js',
-					esModule: true,
+					esModule: 'if-default-prop',
 					exports: 'auto',
 					extend: false,
+					externalImportAssertions: true,
 					externalLiveBindings: true,
 					format: 'cjs',
 					freeze: true,
@@ -35,7 +37,7 @@ module.exports = {
 						arrowFunctions: false,
 						constBindings: false,
 						objectShorthand: false,
-						reservedNamesAsProps: false,
+						reservedNamesAsProps: true,
 						symbols: false
 					},
 					globals: {},
@@ -53,7 +55,7 @@ module.exports = {
 					sourcemap: false,
 					sourcemapExcludeSources: false,
 					strict: true,
-					systemNullSetters: false,
+					systemNullSetters: true,
 					validate: false
 				});
 				assert.strictEqual(options.banner(), 'exports.bar = 43;');
