@@ -1,4 +1,4 @@
-const assert = require('assert');
+const assert = require('node:assert');
 const {
 	closeSync,
 	fsyncSync,
@@ -8,9 +8,9 @@ const {
 	unlinkSync,
 	writeFileSync,
 	writeSync
-} = require('fs');
-const { basename, join } = require('path');
-const { platform, version } = require('process');
+} = require('node:fs');
+const { basename, join } = require('node:path');
+const { platform, version } = require('node:process');
 const fixturify = require('fixturify');
 const { removeSync } = require('fs-extra');
 
@@ -21,44 +21,32 @@ exports.wait = function wait(ms) {
 };
 
 function normaliseError(error) {
-	delete error.stack;
-	delete error.toString;
-	if (error.watchFiles) {
-		error.watchFiles.sort();
+	const clone = { ...error, message: error.message };
+	delete clone.stack;
+	delete clone.toString;
+	if (clone.watchFiles) {
+		clone.watchFiles.sort();
 	}
-	return { ...error, message: error.message };
+	if (clone.frame) {
+		clone.frame = clone.frame.replace(/\s+$/gm, '');
+	}
+	if (clone.cause) {
+		clone.cause = normaliseError(clone.cause);
+	}
+	return clone;
 }
 
 exports.compareError = function compareError(actual, expected) {
 	actual = normaliseError(actual);
-
-	if (actual.parserError) {
-		actual.parserError = normaliseError(actual.parserError);
-	}
-
-	if (actual.frame) {
-		actual.frame = actual.frame.replace(/\s+$/gm, '');
-	}
-
 	if (expected.frame) {
 		expected.frame = deindent(expected.frame);
 	}
-
 	assert.deepEqual(actual, expected);
 };
 
 exports.compareWarnings = function compareWarnings(actual, expected) {
 	assert.deepEqual(
-		actual.map(warning => {
-			const clone = { ...warning };
-			delete clone.toString;
-
-			if (clone.frame) {
-				clone.frame = clone.frame.replace(/\s+$/gm, '');
-			}
-
-			return clone;
-		}),
+		actual.map(normaliseError),
 		expected.map(warning => {
 			if (warning.frame) {
 				warning.frame = deindent(warning.frame);
@@ -68,8 +56,8 @@ exports.compareWarnings = function compareWarnings(actual, expected) {
 	);
 };
 
-function deindent(str) {
-	return str.slice(1).replace(/^\t+/gm, '').replace(/\s+$/gm, '').trim();
+function deindent(stringValue) {
+	return stringValue.slice(1).replace(/^\t+/gm, '').replace(/\s+$/gm, '').trim();
 }
 
 exports.deindent = deindent;
@@ -112,13 +100,13 @@ exports.normaliseOutput = function normaliseOutput(code) {
 	return code.toString().trim().replace(/\r\n/g, '\n');
 };
 
-function runTestSuiteWithSamples(suiteName, samplesDir, runTest, onTeardown) {
-	describe(suiteName, () => runSamples(samplesDir, runTest, onTeardown));
+function runTestSuiteWithSamples(suiteName, samplesDirectory, runTest, onTeardown) {
+	describe(suiteName, () => runSamples(samplesDirectory, runTest, onTeardown));
 }
 
 // You can run only or skip certain kinds of tests by appending .only or .skip
-runTestSuiteWithSamples.only = function (suiteName, samplesDir, runTest, onTeardown) {
-	describe.only(suiteName, () => runSamples(samplesDir, runTest, onTeardown));
+runTestSuiteWithSamples.only = function (suiteName, samplesDirectory, runTest, onTeardown) {
+	describe.only(suiteName, () => runSamples(samplesDirectory, runTest, onTeardown));
 };
 
 runTestSuiteWithSamples.skip = function (suiteName) {
@@ -127,43 +115,43 @@ runTestSuiteWithSamples.skip = function (suiteName) {
 
 exports.runTestSuiteWithSamples = runTestSuiteWithSamples;
 
-function runSamples(samplesDir, runTest, onTeardown) {
+function runSamples(samplesDirectory, runTest, onTeardown) {
 	if (onTeardown) {
 		afterEach(onTeardown);
 	}
 
-	readdirSync(samplesDir)
+	for (const fileName of readdirSync(samplesDirectory)
 		.filter(name => name[0] !== '.')
-		.sort()
-		.forEach(fileName => runTestsInDir(join(samplesDir, fileName), runTest));
+		.sort()) {
+		runTestsInDirectory(join(samplesDirectory, fileName), runTest);
+	}
 }
 
-function runTestsInDir(dir, runTest) {
-	const fileNames = getFileNamesAndRemoveOutput(dir);
+function runTestsInDirectory(directory, runTest) {
+	const fileNames = getFileNamesAndRemoveOutput(directory);
 	if (fileNames.includes('_config.js')) {
-		loadConfigAndRunTest(dir, runTest);
+		loadConfigAndRunTest(directory, runTest);
 	} else if (fileNames.length === 0) {
-		console.warn(`Removing empty test directory ${dir}`);
-		removeSync(dir);
+		console.warn(`Removing empty test directory ${directory}`);
+		removeSync(directory);
 	} else {
-		describe(basename(dir), () => {
-			fileNames
-				.filter(name => name[0] !== '.')
-				.sort()
-				.forEach(fileName => runTestsInDir(join(dir, fileName), runTest));
+		describe(basename(directory), () => {
+			for (const fileName of fileNames.filter(name => name[0] !== '.').sort()) {
+				runTestsInDirectory(join(directory, fileName), runTest);
+			}
 		});
 	}
 }
 
-function getFileNamesAndRemoveOutput(dir) {
+function getFileNamesAndRemoveOutput(directory) {
 	try {
-		return readdirSync(dir).filter(fileName => {
+		return readdirSync(directory).filter(fileName => {
 			if (fileName === '_actual') {
-				removeSync(join(dir, '_actual'));
+				removeSync(join(directory, '_actual'));
 				return false;
 			}
 			if (fileName === '_actual.js') {
-				unlinkSync(join(dir, '_actual.js'));
+				unlinkSync(join(directory, '_actual.js'));
 				return false;
 			}
 			return true;
@@ -171,7 +159,7 @@ function getFileNamesAndRemoveOutput(dir) {
 	} catch (error) {
 		if (error.code === 'ENOTDIR') {
 			throw new Error(
-				`${dir} is not located next to a "_config.js" file but is not a directory or old test output either. Please inspect and consider removing the file.`
+				`${directory} is not located next to a "_config.js" file but is not a directory or old test output either. Please inspect and consider removing the file.`
 			);
 		}
 		throw error;
@@ -180,8 +168,8 @@ function getFileNamesAndRemoveOutput(dir) {
 
 exports.getFileNamesAndRemoveOutput = getFileNamesAndRemoveOutput;
 
-function loadConfigAndRunTest(dir, runTest) {
-	const configFile = join(dir, '_config.js');
+function loadConfigAndRunTest(directory, runTest) {
+	const configFile = join(directory, '_config.js');
 	const config = require(configFile);
 	if (!config || !config.description) {
 		throw new Error(`Found invalid config without description: ${configFile}`);
@@ -191,27 +179,31 @@ function loadConfigAndRunTest(dir, runTest) {
 		(!config.onlyWindows || platform === 'win32') &&
 		(!config.minNodeVersion || config.minNodeVersion <= Number(/^v(\d+)/.exec(version)[1]))
 	) {
-		runTest(dir, config);
+		runTest(directory, config);
 	}
 }
 
-exports.assertDirectoriesAreEqual = function assertDirectoriesAreEqual(actualDir, expectedDir) {
-	const actualFiles = fixturify.readSync(actualDir);
+exports.assertDirectoriesAreEqual = function assertDirectoriesAreEqual(
+	actualDirectory,
+	expectedDirectory
+) {
+	const actualFiles = fixturify.readSync(actualDirectory);
 
 	let expectedFiles;
 	try {
-		expectedFiles = fixturify.readSync(expectedDir);
-	} catch (err) {
+		expectedFiles = fixturify.readSync(expectedDirectory);
+	} catch {
 		expectedFiles = [];
 	}
 	assertFilesAreEqual(actualFiles, expectedFiles);
 };
 
-function assertFilesAreEqual(actualFiles, expectedFiles, dirs = []) {
-	Object.keys({ ...actualFiles, ...expectedFiles }).forEach(fileName => {
-		const pathSegments = dirs.concat(fileName);
+function assertFilesAreEqual(actualFiles, expectedFiles, directories = []) {
+	for (const fileName of Object.keys({ ...actualFiles, ...expectedFiles })) {
+		const pathSegments = [...directories, fileName];
 		if (typeof actualFiles[fileName] === 'object' && typeof expectedFiles[fileName] === 'object') {
-			return assertFilesAreEqual(actualFiles[fileName], expectedFiles[fileName], pathSegments);
+			assertFilesAreEqual(actualFiles[fileName], expectedFiles[fileName], pathSegments);
+			continue;
 		}
 
 		const shortName = pathSegments.join('/');
@@ -219,7 +211,7 @@ function assertFilesAreEqual(actualFiles, expectedFiles, dirs = []) {
 			`${shortName}: ${actualFiles[fileName]}`,
 			`${shortName}: ${expectedFiles[fileName]}`
 		);
-	});
+	}
 }
 
 exports.assertFilesAreEqual = assertFilesAreEqual;
@@ -230,10 +222,10 @@ exports.assertIncludes = function assertIncludes(actual, expected) {
 			actual.includes(expected),
 			`${JSON.stringify(actual)}\nshould include\n${JSON.stringify(expected)}`
 		);
-	} catch (err) {
-		err.actual = actual;
-		err.expected = expected;
-		throw err;
+	} catch (error) {
+		error.actual = actual;
+		error.expected = expected;
+		throw error;
 	}
 };
 
@@ -243,10 +235,10 @@ exports.assertDoesNotInclude = function assertDoesNotInclude(actual, expected) {
 			!actual.includes(expected),
 			`${JSON.stringify(actual)}\nshould not include\n${JSON.stringify(expected)}`
 		);
-	} catch (err) {
-		err.actual = actual;
-		err.expected = expected;
-		throw err;
+	} catch (error) {
+		error.actual = actual;
+		error.expected = expected;
+		throw error;
 	}
 };
 
@@ -261,7 +253,7 @@ function atomicWriteFileSync(filePath, contents) {
 
 exports.atomicWriteFileSync = atomicWriteFileSync;
 
-// It appears that on MacOS, it sometimes takes long for the file system to update
+// It appears that on macOS, it sometimes takes long for the file system to update
 exports.writeAndSync = function writeAndSync(filePath, contents) {
 	const file = openSync(filePath, 'w');
 	writeSync(file, contents);
@@ -269,10 +261,10 @@ exports.writeAndSync = function writeAndSync(filePath, contents) {
 	closeSync(file);
 };
 
-// Sometimes, watchers on MacOS do not seem to fire. In those cases, it helps
+// Sometimes, watchers on macOS do not seem to fire. In those cases, it helps
 // to write the same content again. This function returns a callback to stop
 // further updates.
-function writeAndRetry(filePath, contents) {
+exports.writeAndRetry = function writeAndRetry(filePath, contents) {
 	let retries = 0;
 	let updateRetryTimeout;
 
@@ -287,6 +279,17 @@ function writeAndRetry(filePath, contents) {
 
 	writeFile();
 	return () => clearTimeout(updateRetryTimeout);
-}
+};
 
-exports.writeAndRetry = writeAndRetry;
+exports.replaceDirectoryInStringifiedObject = function replaceDirectoryInStringifiedObject(
+	object,
+	replaced
+) {
+	return JSON.stringify(object, null, 2).replace(
+		new RegExp(
+			JSON.stringify(JSON.stringify(replaced).slice(1, -1)).slice(1, -1) + '[/\\\\]*',
+			'g'
+		),
+		'**/'
+	);
+};
